//
// Funktionen fuer einen binaeren Suchbaum
//

#include "btree.h"


///////////////////////////////////////////////////////////////////////////////
// find
// such das Element mit dem Namen und liefert einen Zeiger darauf zurueck
// oder NULL
///////////////////////////////////////////////////////////////////////////////

struct btree_node *tree_find(struct btree *tree, const char *name)
{
if(tree->root==NULL)
	{
	//printf("empty list, nothing to find...\n");
	return NULL;
	}

struct btree_node *node;
node=tree->root;

while(node!=NULL)
	{
	int comp=strcmp(node->name,name);
	//printf("tree_find: comp (%s,%s)=%d\n",node->name,name,comp);
	if(comp==0)
		{
		return node;
		}
	else if(comp<0)
		{
		node=node->left;
		}
	else
		{
		node=node->right;
		}
	}

return NULL;
}

///////////////////////////////////////////////////////////////////////////////
// add
// fuegt ein Element in den Baum ein
///////////////////////////////////////////////////////////////////////////////

int tree_add(struct btree *tree, const char *name, int stamp)
{
struct btree_node *ptr=NULL;
int done=0;

// Element einraeumen
//
if(tree->root==NULL)
	{
	done=-1;
	}
else
	{
	ptr=tree->root;

	done=0;
	while(done==0)
		{
		int comp=strcmp(ptr->name,name);
		//printf("tree_add: strcmp(%s,%s)=%d\n",iter->name,node->name,comp);
		if(comp==0)
			{
			done=1;
			}
		else if(comp<0)
			{
			if(ptr->left==NULL)
				{
				done=2;
				}
			else
				{
				ptr=ptr->left;
				}
			}
		else
			{
			if(ptr->right==NULL)
				{
				done=3;
				}
			else
				{
				ptr=ptr->right;
				}
			}
		}
	}

// Wenn gefunden, dann nur Aendern
//
if(done==1)
	{
	ptr->stamp=stamp;
	return 0;
	}

// neuen Knoten einfuegen
//

struct btree_node *node;
node=malloc(sizeof(struct btree_node));
if(node==NULL)
	{
	perror("malloc failed");
	return 1;
	}

node->name=calloc(sizeof(char),strlen(name)+1);
if(node->name==NULL)
	{
	perror("calloc failed");
	free(node);
	return 1;
	}
strcpy(node->name,name);
node->stamp=stamp;
node->left=node->right=NULL;

// an richtiger Stelle einsortieren
//

if(done==-1)
	{
	tree->root=node;
	}
else if(done==2)
	{
	ptr->left=node;
	}
else if(done==3)
	{
	ptr->right=node;
	}

return 1;
}



///////////////////////////////////////////////////////////////////////////////
// init
// erzeugt einen leeren Baum
///////////////////////////////////////////////////////////////////////////////

struct btree *tree_init(void)
{
struct btree *tree;
tree=malloc(sizeof(struct btree));
if(tree!=NULL)
	{
	tree->root=NULL;
	return tree;
	}
perror("unable to malloc");
return NULL;
}

///////////////////////////////////////////////////////////////////////////////
// tree_destroy
// gibt den Speicher eines Baums frei
///////////////////////////////////////////////////////////////////////////////

void tree_destroy(struct btree *tree)
{
//struct btree_node *initial_node;

if(tree->root!=NULL)
	{
	tree_destroy_node(tree->root);
	}

free(tree);
tree=NULL;
return;
}

///////////////////////////////////////////////////////////////////////////////
// tree_destroy_node
// gibt den Speicher eines Nodes und aller Subnodes frei
///////////////////////////////////////////////////////////////////////////////

void tree_destroy_node(struct btree_node *node)
{
if(node->left!=NULL)
	{
	tree_destroy_node(node->left);
	}
if(node->right!=NULL)
	{
	tree_destroy_node(node->right);
	}
//printf("freeing node(%s)\n",node->name);
free(node->name);
free(node);
node=NULL;
}
