//
// hash.c
//

#include "hash.h"

///////////////////////////////////////////////////////////////////////////////
// hash_init
// erzeugt ein leeres Hash
///////////////////////////////////////////////////////////////////////////////

struct hash *hash_init(unsigned int slots)
{
struct hash *h=malloc(sizeof(struct hash));
h->slots=slots;
h->elements=0;

struct btree **buckets=(struct btree **)malloc(slots * sizeof(struct btree *));
for(slots=0; slots<h->slots; slots++)
	{
	buckets[slots]=NULL;
	}
h->buckets=buckets;
return h;
}

///////////////////////////////////////////////////////////////////////////////
// hash_set
// fuegt ein Schluessel-Wert-Paar ein oder aendert den Wert eines vorhanden
// Schluessels
///////////////////////////////////////////////////////////////////////////////

int hash_set(struct hash *h, const char *key, unsigned int value)
{
unsigned int slot=hash_decompose(h,key);
//printf("key %s slot %d\n",key,slot);
if(h->buckets[slot]==NULL)
	{
	h->buckets[slot]=tree_init();
	}
int result=tree_add(h->buckets[slot],key,value);
if(result!=0)
	{
	h->elements++;
	}
return result;
}

///////////////////////////////////////////////////////////////////////////////
// hash_get
// liefert den Wert eines Schluessels oder 0
///////////////////////////////////////////////////////////////////////////////

struct btree_node *hash_get(struct hash *h, const char *key)
{
unsigned int slot=hash_decompose(h,key);
//printf("key %s slot %d\n",key,slot);

if(h->buckets[slot]==NULL)
	{
	return NULL;
	}

return tree_find(h->buckets[slot],key);
}

///////////////////////////////////////////////////////////////////////////////
// hash_free
// loescht das komplette Hash
///////////////////////////////////////////////////////////////////////////////

void hash_free(struct hash *h)
{
int slot;
for(slot=0; slot<h->slots; slot++)
	{
	if(h->buckets[slot]!=NULL)
		{
		tree_destroy(h->buckets[slot]);
		}
	}
free(h->buckets);
free(h);
}

///////////////////////////////////////////////////////////////////////////////
// hash_decompose
// baut einen Schluessel in eine Slotnummer um
///////////////////////////////////////////////////////////////////////////////

unsigned int hash_decompose(struct hash *h, const char *key)
{
unsigned int len=strlen(key);
unsigned int slot=key[0];
int i;
for(i=1; i<len; i++)
	{
	slot=((slot*128)+key[i])%(h->slots);
	}
return slot;
}
